@@ -181,12 +181,20 @@ class ShowContainer(command.ShowOne):
             'container',
             metavar='<container>',
             help='ID or name of the container to show.')
+        parser.add_argument(
+            '--all-tenants',
+            action="store_true",
+            default=False,
+            help='Show container(s) in all tenant by name.')
         return parser
 
     def take_action(self, parsed_args):
         client = _get_client(self, parsed_args)
-        container = parsed_args.container
-        container = client.containers.get(container)
+        opts = {}
+        opts['id'] = parsed_args.container
+        opts['all_tenants'] = parsed_args.all_tenants
+        opts = zun_utils.remove_null_parms(**opts)
+        container = client.containers.get(**opts)
         columns = _container_columns(container)
 
         return columns, utils.get_item_properties(container, columns)
@@ -259,15 +267,24 @@ class DeleteContainer(command.Command):
             '--force',
             action='store_true',
             help='Force delete the container.')
+        parser.add_argument(
+            '--all-tenants',
+            action="store_true",
+            default=False,
+            help='Delete container(s) in all tenant by name.')
         return parser
 
     def take_action(self, parsed_args):
         client = _get_client(self, parsed_args)
         containers = parsed_args.container
-        force = getattr(parsed_args, 'force')
         for container in containers:
+            opts = {}
+            opts['id'] = container
+            opts['force'] = parsed_args.force
+            opts['all_tenants'] = parsed_args.all_tenants
+            opts = zun_utils.remove_null_parms(**opts)
             try:
-                client.containers.delete(container, force)
+                client.containers.delete(**opts)
                 print(_('Request to delete container %s has been accepted.')
                       % container)
             except Exception as e:
@@ -399,17 +416,32 @@ class ExecContainer(command.Command):
             metavar='<command>',
             nargs=argparse.REMAINDER,
             help='The command to execute.')
+        parser.add_argument(
+            '--interactive',
+            dest='interactive',
+            action='store_true',
+            default=False,
+            help='Keep STDIN open and allocate a pseudo-TTY for interactive')
         return parser
 
     def take_action(self, parsed_args):
         client = _get_client(self, parsed_args)
         container = parsed_args.container
-        command = zun_utils.parse_command(parsed_args.command)
-        response = client.containers.execute(container, command=command)
-        output = response['output']
-        exit_code = response['exit_code']
-        print(output)
-        return exit_code
+        opts = {}
+        opts['command'] = zun_utils.parse_command(parsed_args.command)
+        if parsed_args.interactive:
+            opts['interactive'] = True
+            opts['run'] = False
+        response = client.containers.execute(container, **opts)
+        if parsed_args.interactive:
+            exec_id = response['exec_id']
+            url = response['url']
+            websocketclient.do_exec(client, url, container, exec_id, "~", 0.5)
+        else:
+            output = response['output']
+            exit_code = response['exit_code']
+            print(output)
+            return exit_code
 
 
 class LogsContainer(command.Command):
@@ -544,10 +576,6 @@ class RunContainer(command.ShowOne):
             'image',
             metavar='<image>',
             help='name or ID of the image')
-        parser.add_argument(
-            '--command',
-            metavar='<command>',
-            help='Send command to the container')
         parser.add_argument(
             '--cpu',
             metavar='<cpu>',
