@@ -52,14 +52,16 @@ try:
 except ImportError:
     pass
 
+from zunclient import api_versions
 from zunclient.common.apiclient import auth
 from zunclient.common import cliutils
 from zunclient import exceptions as exc
+from zunclient.i18n import _
 from zunclient.v1 import client as client_v1
 from zunclient.v1 import shell as shell_v1
 from zunclient import version
 
-DEFAULT_API_VERSION = '1'
+DEFAULT_API_VERSION = '1.2'
 DEFAULT_ENDPOINT_TYPE = 'publicURL'
 DEFAULT_SERVICE_TYPE = 'container'
 
@@ -332,8 +334,8 @@ class OpenStackZunShell(object):
                             default=cliutils.env(
                                 'ZUN_API_VERSION',
                                 default=DEFAULT_API_VERSION),
-                            help='Accepts "api", '
-                                 'defaults to env[ZUN_API_VERSION].')
+                            help='Accepts X, X.Y (where X is major, Y is minor'
+                                 ' part), defaults to env[ZUN_API_VERSION].')
         parser.add_argument('--zun_api_version',
                             help=argparse.SUPPRESS)
 
@@ -362,11 +364,13 @@ class OpenStackZunShell(object):
         if profiler:
             parser.add_argument('--profile',
                                 metavar='HMAC_KEY',
+                                default=cliutils.env('OS_PROFILE',
+                                                     default=None),
                                 help='HMAC key to use for encrypting context '
                                      'data for performance profiling of '
                                      'operation. This key should be the '
                                      'value of the HMAC key configured for '
-                                     'the OSprofiler middleware in nova; it '
+                                     'the OSprofiler middleware in zun; it '
                                      'is specified in the Zun configuration '
                                      'file at "/etc/zun/zun.conf". Without '
                                      'the key, profiling functions will not '
@@ -378,7 +382,7 @@ class OpenStackZunShell(object):
 
         return parser
 
-    def get_subcommand_parser(self, version):
+    def get_subcommand_parser(self, version, do_help=False):
         parser = self.get_base_parser()
 
         self.subcommands = {}
@@ -386,14 +390,14 @@ class OpenStackZunShell(object):
 
         try:
             actions_modules = {
-                '1': shell_v1.COMMAND_MODULES,
-            }[version]
+                '1': shell_v1.COMMAND_MODULES
+            }[version.ver_major]
         except KeyError:
             actions_modules = shell_v1.COMMAND_MODULES
 
         for actions_module in actions_modules:
-            self._find_actions(subparsers, actions_module)
-        self._find_actions(subparsers, self)
+            self._find_actions(subparsers, actions_module, version, do_help)
+        self._find_actions(subparsers, self, version, do_help)
 
         self._add_bash_completion_subparser(subparsers)
 
@@ -408,12 +412,27 @@ class OpenStackZunShell(object):
         self.subcommands['bash_completion'] = subparser
         subparser.set_defaults(func=self.do_bash_completion)
 
-    def _find_actions(self, subparsers, actions_module):
+    def _find_actions(self, subparsers, actions_module, version, do_help):
+        msg = _(" (Supported by API versions '%(start)s' - '%(end)s')")
         for attr in (a for a in dir(actions_module) if a.startswith('do_')):
             # I prefer to be hyphen-separated instead of underscores.
             command = attr[3:].replace('_', '-')
             callback = getattr(actions_module, attr)
             desc = callback.__doc__ or ''
+            if hasattr(callback, "versioned"):
+                subs = api_versions.get_substitutions(callback)
+                if do_help:
+                    desc += msg % {'start': subs[0].start_version.get_string(),
+                                   'end': subs[-1].end_version.get_string()}
+                else:
+                    for versioned_method in subs:
+                        if version.matches(versioned_method.start_version,
+                                           versioned_method.end_version):
+                            callback = versioned_method.func
+                            break
+                    else:
+                        continue
+
             action_help = desc.strip()
             arguments = getattr(callback, 'arguments', [])
 
@@ -430,6 +449,25 @@ class OpenStackZunShell(object):
             self.subcommands[command] = subparser
 
             for (args, kwargs) in arguments:
+                start_version = kwargs.get("start_version", None)
+                if start_version:
+                    start_version = api_versions.APIVersion(start_version)
+                    end_version = kwargs.get("end_version", None)
+                    if end_version:
+                        end_version = api_versions.APIVersion(end_version)
+                    else:
+                        end_version = api_versions.APIVersion(
+                            "%s.latest" % start_version.ver_major)
+                    if do_help:
+                        kwargs["help"] = kwargs.get("help", "") + (msg % {
+                            "start": start_version.get_string(),
+                            "end": end_version.get_string()})
+                    else:
+                        if not version.matches(start_version, end_version):
+                            continue
+                kw = kwargs.copy()
+                kw.pop("start_version", None)
+                kw.pop("end_version", None)
                 subparser.add_argument(*args, **kwargs)
             subparser.set_defaults(func=callback)
 
@@ -456,6 +494,8 @@ class OpenStackZunShell(object):
         (options, args) = parser.parse_known_args(argv)
         self.setup_debugging(options.debug)
 
+        api_version = api_versions.get_api_version(options.zun_api_version)
+
         # NOTE(dtroyer): Hackery to handle --endpoint_type due to argparse
         #                thinking usage-list --end is ambiguous; but it
         #                works fine with only --endpoint-type present
@@ -464,9 +504,9 @@ class OpenStackZunShell(object):
             spot = argv.index('--endpoint_type')
             argv[spot] = '--endpoint-type'
 
-        subcommand_parser = (
-            self.get_subcommand_parser(options.zun_api_version)
-        )
+        subcommand_parser = self.get_subcommand_parser(
+            api_version, do_help=("help" in args))
+
         self.parser = subcommand_parser
 
         if options.help or not argv:
@@ -571,7 +611,7 @@ class OpenStackZunShell(object):
         try:
             client = {
                 '1': client_v1,
-            }[options.zun_api_version]
+            }[api_version.ver_major]
         except KeyError:
             client = client_v1
 
@@ -593,6 +633,7 @@ class OpenStackZunShell(object):
                                 zun_url=bypass_url,
                                 endpoint_type=endpoint_type,
                                 insecure=insecure,
+                                api_version=api_version,
                                 **kwargs)
 
         args.func(self.cs, args)
